from __future__ import annotations

import asyncio
import base64
import json
import logging
import os
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple, Union
from datetime import datetime

from openai import OpenAI

from .exceptions import ModelNotFoundError, ConfigurationError
from .utils import Config
from .utils.config import ModelType, Provider, ModelConfig

class LLMClient:
    """LLM 客户端基类，提供核心的模型交互功能
    
    Examples:
        >>> client = LLMClient()
        >>> client.set_model("llm", "openai", "gpt-4")
        >>> response = await client.chat("Hello!")
        
        >>> # 使用视觉模型
        >>> client.set_model("vlm", "openai", "gpt-4-vision")
        >>> response = await client.chat("描述这张图片", images=["image.jpg"])
    """
    
    DEFAULT_MAX_TOKENS = 8192

    def __init__(self, config_path: Optional[str] = None):
        """初始化客户端
        
        Args:
            config_path: 配置文件路径，如果不指定则使用默认路径
        """
        self.config_path = Path(config_path) if config_path else Path.home() / ".think_llm_client" / "config.json"
        self.history_dir = self.config_path.parent / "history"
        self.history_dir.mkdir(parents=True, exist_ok=True)
        
        self.model_types: Dict[str, ModelType] = {}
        self.current_model_type: Optional[str] = None
        self.current_provider: Optional[str] = None
        self.current_model: Optional[str] = None
        self.client: Optional[OpenAI] = None
        self.messages: List[Dict[str, Any]] = []  
        self.system_prompt: Optional[str] = None
        
        self._load_config()

    def _load_config(self) -> None:
        """加载配置文件"""
        try:
            config = Config(self.config_path)
            
            # 转换配置数据结构
            self.model_types = {
                type_name: ModelType.from_dict(type_data)
                for type_name, type_data in config.config.items()
            }
            
        except Exception as e:
            logging.error(f"加载配置文件失败: {e}")
            raise ConfigurationError(f"无法加载配置文件: {e}")

    def get_available_models(self) -> List[Tuple[str, str, str]]:
        """获取可用的模型列表
        
        Returns:
            List[Tuple[str, str, str]]: 返回模型列表，每个元素为 (model_type, provider_key, model_key)
        """
        models = []
        for type_key, type_info in self.model_types.items():
            for provider_key, provider in type_info.providers.items():
                for model_key in provider.model.keys():
                    models.append((type_key, provider_key, model_key))
        return models

    def set_model(self, model_type: str, provider_key: str, model_key: str) -> None:
        """设置当前使用的模型
        
        Args:
            model_type: 模型类型
            provider_key: 供应商标识
            model_key: 模型标识
            
        Raises:
            ValueError: 当指定的模型类型、供应商或模型不存在时抛出
        """
        if model_type not in self.model_types:
            raise ValueError(f"未知的模型类型: {model_type}")
        
        model_type_config = self.model_types[model_type]
        if provider_key not in model_type_config.providers:
            raise ValueError(f"未知的供应商: {provider_key}")
            
        provider = model_type_config.providers[provider_key]
        if model_key not in provider.model:
            raise ValueError(f"未知的模型: {model_key}")
        
        self.current_model_type = model_type
        self.current_provider = provider_key
        self.current_model = model_key
        
        self.client = OpenAI(
            api_key=provider.api_key,
            base_url=provider.api_url
        )

    async def chat_stream(
        self, 
        message: str, 
        images: Optional[List[str]] = None
    ):
        """流式对话方法
        
        Args:
            message: 用户输入的文本消息
            images: 可选的图片路径列表。当使用 VLM 类型模型时，可以传入一个或多个图片路径
            
        Yields:
            Tuple[str, str, str]: (type, content, full_content)
            - type: "reasoning" 或 "content"
            - content: 当前 chunk 的内容
            - full_content: 到目前为止收到的所有内容的合并
        """
        if not all([self.current_model_type, self.current_provider, self.current_model, self.client]):
            raise RuntimeError("未设置模型")

        provider = self.model_types[self.current_model_type].providers[self.current_provider]
        model_config = provider.model[self.current_model]

        # 构建消息历史
        messages = []
        if self.system_prompt:
            messages.append({"role": "system", "content": self.system_prompt})
        
        messages.extend(self.messages)

        if self.current_model_type == "vlm":
            content = [{"type": "text", "text": message}]
            
            if images:
                for image_path in images:
                    if not os.path.exists(image_path):
                        logging.warning(f"图片不存在: {image_path}")
                        continue
                    try:
                        image_type = os.path.splitext(image_path)[1][1:].lower()
                        if image_type not in ['jpg', 'jpeg', 'png', 'gif', 'webp']:
                            image_type = 'jpeg'
                        
                        with open(image_path, "rb") as f:
                            import base64
                            image_base64 = base64.b64encode(f.read()).decode('utf-8')
                            content.append({
                                "type": "image_url",
                                "image_url": {
                                    "url": f"data:image/{image_type};base64,{image_base64}"
                                }
                            })
                    except Exception as e:
                        logging.error(f"读取图片失败 {image_path}: {e}")
                        continue
            
            messages.append({"role": "user", "content": content})
        else:
            messages.append({"role": "user", "content": message})

        try:
            # 保存用户消息到历史记录
            self.messages.append({
                "role": "user",
                "content": message
            })

            response = await asyncio.to_thread(
                self.client.chat.completions.create,
                model=self.current_model,
                messages=messages,
                max_tokens=model_config.max_tokens,
                stream=True
            )

            reasoning_chunks = []
            content_chunks = []

            for chunk in response:
                # 处理推理模型的思维链输出
                if hasattr(chunk.choices[0].delta, 'reasoning_content') and chunk.choices[0].delta.reasoning_content:
                    current_reasoning = chunk.choices[0].delta.reasoning_content
                    if current_reasoning:
                        reasoning_chunks.append(current_reasoning)
                        yield "reasoning", current_reasoning, ''.join(reasoning_chunks)

                # 处理内容输出
                if hasattr(chunk.choices[0].delta, 'content') and chunk.choices[0].delta.content:
                    current_chunk = chunk.choices[0].delta.content
                    if current_chunk:
                        content_chunks.append(current_chunk)
                        yield "content", current_chunk, ''.join(content_chunks)

            # 保存完整的助手回复到历史记录
            content = ''.join(content_chunks)
            reasoning_content = ''.join(reasoning_chunks)
            if content:  # 只有在有内容时才保存助手回复
                self.messages.append({
                    "role": "assistant",
                    "content": content,
                    "reasoning_content": reasoning_content if self.current_model_type == "reasoning" else None
                })

        except Exception as e:
            logging.error(f"LLM 请求失败: {e}")

    async def chat(
        self, 
        message: str, 
        images: Optional[List[str]] = None,
        stream: bool = True
    ) -> Tuple[Optional[str], Optional[str]]:
        """与 LLM 进行对话
        
        Args:
            message: 用户输入的文本消息
            images: 可选的图片路径列表。当使用 VLM 类型模型时，可以传入一个或多个图片路径
            stream: 是否使用流式输出
            
        Returns:
            Tuple[Optional[str], Optional[str]]: (reasoning_content, content)
        """
        if stream:
            # 使用流式输出
            reasoning_chunks = []
            content_chunks = []
            
            try:
                async for chunk_type, chunk, _ in self.chat_stream(message, images):
                    if chunk_type == "reasoning":
                        reasoning_chunks.append(chunk)
                    elif chunk_type == "content":
                        content_chunks.append(chunk)
                
                return ''.join(reasoning_chunks), ''.join(content_chunks)
            except Exception as e:
                logging.error(f"流式对话失败: {e}")
                return None, None
        else:
            # 使用非流式输出
            if not all([self.current_model_type, self.current_provider, self.current_model, self.client]):
                raise RuntimeError("未设置模型")

            provider = self.model_types[self.current_model_type].providers[self.current_provider]
            model_config = provider.model[self.current_model]

            # 构建消息历史
            messages = []
            if self.system_prompt:
                messages.append({"role": "system", "content": self.system_prompt})
            
            messages.extend(self.messages)

            if self.current_model_type == "vlm":
                content = [{"type": "text", "text": message}]
                
                if images:
                    for image_path in images:
                        if not os.path.exists(image_path):
                            logging.warning(f"图片不存在: {image_path}")
                            continue
                        try:
                            image_type = os.path.splitext(image_path)[1][1:].lower()
                            if image_type not in ['jpg', 'jpeg', 'png', 'gif', 'webp']:
                                image_type = 'jpeg'
                            
                            with open(image_path, "rb") as f:
                                import base64
                                image_base64 = base64.b64encode(f.read()).decode('utf-8')
                                content.append({
                                    "type": "image_url",
                                    "image_url": {
                                        "url": f"data:image/{image_type};base64,{image_base64}"
                                    }
                                })
                        except Exception as e:
                            logging.error(f"读取图片失败 {image_path}: {e}")
                            continue
                
                messages.append({"role": "user", "content": content})
            else:
                messages.append({"role": "user", "content": message})

            try:
                # 保存用户消息到历史记录
                self.messages.append({
                    "role": "user",
                    "content": message
                })

                response = await asyncio.to_thread(
                    self.client.chat.completions.create,
                    model=self.current_model,
                    messages=messages,
                    max_tokens=model_config.max_tokens,
                    stream=False
                )

                # 从响应中提取内容
                content = response.choices[0].message.content
                reasoning_content = getattr(response.choices[0].message, 'reasoning_content', None)

                # 保存助手回复到历史记录
                if content:
                    self.messages.append({
                        "role": "assistant",
                        "content": content,
                        "reasoning_content": reasoning_content if self.current_model_type == "reasoning" else None
                    })

                return reasoning_content, content

            except Exception as e:
                logging.error(f"LLM 请求失败: {e}")
                return None, None

    def clear_history(self) -> None:
        """清除对话历史"""
        self.messages = []

    def load_history(self, messages: List[Dict[str, Any]]) -> None:
        """加载对话历史"""
        self.messages = messages  

    def get_available_histories(self) -> List[Tuple[Path, str]]:
        """获取可用的对话历史记录
        
        Returns:
            List[Tuple[Path, str]]: 返回历史记录列表，每个元素为 (文件路径, 时间戳)
        """
        histories = []
        for file in self.history_dir.glob("chat_*.json"):
            try:
                with open(file, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    timestamp = data.get('timestamp', '')
                    histories.append((file, timestamp))
            except:
                continue
        return sorted(histories, key=lambda x: x[1], reverse=True)

    def save_chat_history(self, filepath: Optional[Union[str, Path]] = None) -> bool:
        """保存当前对话历史
        
        Args:
            filepath: 保存的文件路径，如果不指定则使用默认路径（history_dir/chat_YYYYMMDD_HHMMSS.json）
            
        Returns:
            bool: 是否保存成功
        """
        try:
            if not filepath:
                timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
                filepath = self.history_dir / f"chat_{timestamp}.json"
            else:
                filepath = Path(filepath)
                
            filepath.parent.mkdir(parents=True, exist_ok=True)
            
            history = {
                "timestamp": datetime.now().strftime("%Y%m%d_%H%M%S"),
                "messages": self.messages,  
                "system_prompt": self.system_prompt,
                "model": self.current_model,
                "max_tokens": self.model_types[self.current_model_type].providers[self.current_provider].model[self.current_model].max_tokens if all([self.current_model_type, self.current_provider, self.current_model]) else None
            }
            
            with open(filepath, 'w', encoding='utf-8') as f:
                json.dump(history, f, ensure_ascii=False, indent=2)
            return True
            
        except Exception as e:
            logging.error(f"保存对话历史失败: {e}")
            return False

    def load_chat_history_from_file(self, filepath: Union[str, Path]) -> bool:
        """从文件加载对话历史
        
        Args:
            filepath: 历史文件路径
            
        Returns:
            bool: 是否加载成功
        """
        try:
            filepath = Path(filepath)
            if not filepath.exists():
                return False
                
            with open(filepath, 'r', encoding='utf-8') as f:
                data = json.load(f)
                
            self.messages = data['messages']  
            if data.get('system_prompt'):
                self.system_prompt = data['system_prompt']
                
            return True
            
        except Exception as e:
            logging.error(f"加载对话历史失败: {e}")
            return False

    async def analyze_image(self, image_path: str, prompt: Optional[str] = None) -> Tuple[Optional[str], Optional[str]]:
        """分析单张图片的便捷方法
        
        Args:
            image_path: 图片路径
            prompt: 可选的提示词，默认为"描述这张图片"
            
        Returns:
            Tuple[Optional[str], Optional[str]]: (reasoning_content, content)
        """
        return await self.chat(
            prompt or "描述这张图片",
            images=[image_path]
        )

    async def compare_images(
        self, 
        image_paths: List[str], 
        prompt: Optional[str] = None
    ) -> Tuple[Optional[str], Optional[str]]:
        """比较多张图片的便捷方法
        
        Args:
            image_paths: 图片路径列表
            prompt: 可选的提示词，默认为"比较这些图片的异同"
            
        Returns:
            Tuple[Optional[str], Optional[str]]: (reasoning_content, content)
        """
        return await self.chat(
            prompt or "比较这些图片的异同",
            images=image_paths
        )
